package com.ningdatech.pmapi.performance.helper.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.ningdatech.pmapi.performance.enumration.AppraisalTypeEnum;
import com.ningdatech.pmapi.performance.enumration.PerformanceTemplateTypeEnum;
import com.ningdatech.pmapi.performance.helper.TemplateDetailBuildHelper;
import com.ningdatech.pmapi.performance.model.entity.PerformanceAppraisalScoreInfo;
import com.ningdatech.pmapi.performance.model.entity.PerformanceIndicatorProjectTemplateDetail;
import com.ningdatech.pmapi.performance.model.vo.ProjectTemplateDetailVO;
import com.ningdatech.pmapi.performance.service.IPerformanceAppraisalScoreInfoService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author CMM
 * @since 2023/08/09 22:31
 */
@Component
@RequiredArgsConstructor
public class TemplateDetailBuildHelperImpl implements TemplateDetailBuildHelper {

    private final IPerformanceAppraisalScoreInfoService performanceAppraisalScoreInfoService;

    @Override
    public List<ProjectTemplateDetailVO> buildTemplateDetail(List<PerformanceIndicatorProjectTemplateDetail> templateDetails) {

        List<ProjectTemplateDetailVO> res = Lists.newArrayList();
        //第一层是 一级的
        for (PerformanceIndicatorProjectTemplateDetail templateDetailFirst : templateDetails) {
            if (PerformanceTemplateTypeEnum.FIRST_INDEX.getCode().equals(templateDetailFirst.getType())) {
                ProjectTemplateDetailVO detailFirstVo = BeanUtil.copyProperties(templateDetailFirst, ProjectTemplateDetailVO.class);
                List<ProjectTemplateDetailVO> firstList = Lists.newArrayList();
                // 该层循环筛选出第二 指标
                for (PerformanceIndicatorProjectTemplateDetail templateDetailSecond : templateDetails) {
                    if (Objects.nonNull(templateDetailSecond.getParentId()) &&
                            templateDetailSecond.getParentId().equals(templateDetailFirst.getId())) {
                        ProjectTemplateDetailVO detailSecondVo = BeanUtil.copyProperties(templateDetailSecond, ProjectTemplateDetailVO.class);
                        List<ProjectTemplateDetailVO> secondList = Lists.newArrayList();
                        // 该层循环筛选出第三 指标
                        for (PerformanceIndicatorProjectTemplateDetail templateDetailThird : templateDetails) {
                            if (Objects.nonNull(templateDetailThird.getParentId()) &&
                                    templateDetailThird.getParentId().equals(templateDetailSecond.getId())) {
                                ProjectTemplateDetailVO detailThirdVo = BeanUtil.copyProperties(templateDetailThird, ProjectTemplateDetailVO.class);
                                // 如果有单位自评、复评、核查分数信息，各自装配信息
                                // 获取自评分数信息
                                List<PerformanceAppraisalScoreInfo> selfAppraisalScoreInfo = performanceAppraisalScoreInfoService.list(Wrappers.lambdaQuery(PerformanceAppraisalScoreInfo.class)
                                        .eq(PerformanceAppraisalScoreInfo::getTemplateDetailId, templateDetailThird.getId())
                                        .eq(PerformanceAppraisalScoreInfo::getAppraisalType, AppraisalTypeEnum.SELF_APPRAISAL.getCode()));
                                if (CollUtil.isNotEmpty(selfAppraisalScoreInfo)) {
                                    detailThirdVo.setSelfAppraisalScoreInfo(selfAppraisalScoreInfo);
                                }
                                // 按照复评人员分组 获取复评分数信息
                                List<PerformanceAppraisalScoreInfo> reAppraisalScoreInfo = performanceAppraisalScoreInfoService.list(Wrappers.lambdaQuery(PerformanceAppraisalScoreInfo.class)
                                        .eq(PerformanceAppraisalScoreInfo::getTemplateDetailId, templateDetailThird.getId())
                                        .eq(PerformanceAppraisalScoreInfo::getAppraisalType, AppraisalTypeEnum.RE_APPRAISAL.getCode()));
                                if (CollUtil.isNotEmpty(reAppraisalScoreInfo)) {
                                    Map<String, List<PerformanceAppraisalScoreInfo>> reAppScoreInfoMap = reAppraisalScoreInfo.stream().collect(Collectors.groupingBy(PerformanceAppraisalScoreInfo::getAppraisalEmployeeName));
                                    detailThirdVo.setReAppraisalScoreInfo(reAppScoreInfoMap);
                                }
                                // 按照核查人员分组 获取核查分数信息
                                List<PerformanceAppraisalScoreInfo> verifyScoreInfo = performanceAppraisalScoreInfoService.list(Wrappers.lambdaQuery(PerformanceAppraisalScoreInfo.class)
                                        .eq(PerformanceAppraisalScoreInfo::getTemplateDetailId, templateDetailThird.getId())
                                        .eq(PerformanceAppraisalScoreInfo::getAppraisalType, AppraisalTypeEnum.EXPERT_VERIFY.getCode()));
                                if (CollUtil.isNotEmpty(verifyScoreInfo)) {
                                    Map<String, List<PerformanceAppraisalScoreInfo>> verifyScoreInfoMap = verifyScoreInfo.stream().collect(Collectors.groupingBy(PerformanceAppraisalScoreInfo::getAppraisalEmployeeName));
                                    detailThirdVo.setVerifyScoreInfo(verifyScoreInfoMap);
                                }
                                secondList.add(detailThirdVo);
                            }
                        }
                        detailSecondVo.setChildren(secondList);
                        firstList.add(detailSecondVo);
                    }
                }
                detailFirstVo.setChildren(firstList);
                res.add(detailFirstVo);

            }
        }
        return res;
    }
}
